@@ -1,10 +1,41 @@
 from __future__ import annotations
 
+import asyncio
+import threading
 from datetime import date as Date, datetime as Datetime, time as Time
 from itertools import count, islice
 from time import localtime
 from warnings import warn
 
+import asgiref.sync
+
+
+# from asgiref.sync import async_to_sync
+# global loop
+
+def async_to_sync(coroutine, loop=None):
+    if loop is None:
+        try:
+            loop = asyncio.get_running_loop()
+        except RuntimeError:
+            loop = asyncio.new_event_loop()
+
+    def synchronous(*args, **kwargs):
+        # local = threading.local()
+        # if not hasattr(local, 'loop'):
+        #     local.loop = asyncio.new_event_loop()
+        # elif local.loop.is_closed():
+        #     local.loop = asyncio.new_event_loop()
+        # loop = local.loop
+        # try:
+        #     loop = asyncio.get_running_loop()
+        #     print(f"Found an existing event loop ({id(loop)})")
+        # except RuntimeError:
+        #     loop = asyncio.new_event_loop()
+        #     print(f"Creating a new event loop ({id(loop)})")
+        return loop.run_until_complete(coroutine(*args, **kwargs))
+    return synchronous
+
 from pgwasm.converters import (
     BIGINT,
     BOOLEAN,
@@ -189,7 +220,7 @@ def Binary(value):
     return value
 
 
-async def connect(
+async def aconnect(
         user,
         uri: str,
         database=None,
@@ -208,6 +239,8 @@ async def connect(
     await conn.connect()
     return conn
 
+connect = async_to_sync(aconnect)
+
 
 apilevel = "2.0"
 """The DBAPI level supported, currently "2.0".
@@ -369,12 +402,14 @@ def convert_paramstyle(style, query, args):
 class Cursor:
     def __init__(self, connection: Connection):
         self._c = connection
+        self._loop = connection._loop
         self.arraysize = 1
-
         self._context = None
         self._row_iter = None
-
         self._input_oids = ()
+        self.execute = async_to_sync(self.aexecute, self._loop)
+        self.executemany = async_to_sync(self.aexecutemany, self._loop)
+        self.callproc = async_to_sync(self.acallproc, self._loop)
 
     @property
     def connection(self):
@@ -410,7 +445,7 @@ class Cursor:
     # or mapping and will be bound to variables in the operation.
     # <p>
     # Stability: Part of the DBAPI 2.0 specification.
-    async def execute(self, operation, args=(), stream=None):
+    async def aexecute(self, operation, args=(), stream=None):
         """Executes a database operation.  Parameters may be provided as a
         sequence, or as a mapping, depending upon the value of
         :data:`pgwasm.paramstyle`.
@@ -464,7 +499,8 @@ class Cursor:
 
         self.input_types = []
 
-    async def executemany(self, operation, param_sets):
+
+    async def aexecutemany(self, operation, param_sets):
         """Prepare a database operation, and then execute it against all
         parameter sequences or mappings provided.
 
@@ -482,7 +518,7 @@ class Cursor:
         input_oids = self._input_oids
         for parameters in param_sets:
             self._input_oids = input_oids
-            await self.execute(operation, parameters)
+            await self.aexecute(operation, parameters)
             rowcounts.append(self._context.row_count)
 
         if len(rowcounts) == 0:
@@ -492,7 +528,7 @@ class Cursor:
         else:
             self._context.row_count = sum(rowcounts)
 
-    async def callproc(self, procname, parameters=None):
+    async def acallproc(self, procname, parameters=None):
         args = [] if parameters is None else parameters
         operation = f"CALL {procname}(" + ", ".join(["%s" for _ in args]) + ")"
 
@@ -541,7 +577,7 @@ class Cursor:
 
     def __next__(self):
         try:
-            return next(self._row_iter)
+            return tuple(next(self._row_iter))
         except AttributeError:
             if self._context is None:
                 raise ProgrammingError("A query hasn't been issued.")
@@ -625,6 +661,20 @@ class Cursor:
 
 
 class Connection(CoreConnection):
+    def __init__(self, *args, **kwargs):
+        try:
+            self._loop = asyncio.get_running_loop()
+        except RuntimeError:
+            self._loop = asyncio.new_event_loop()
+        self.commit = async_to_sync(self.acommit, self._loop)
+        self.rollback = async_to_sync(self.arollback, self._loop)
+        self.close = async_to_sync(lambda: CoreConnection.close(self), self._loop)
+        self.tpc_begin = async_to_sync(self.atpc_begin, self._loop)
+        self.tpc_prepare = async_to_sync(self.atpc_prepare, self._loop)
+        self.tpc_commit = async_to_sync(self.atpc_commit, self._loop)
+        self.tpc_rollback = async_to_sync(self.atpc_rollback, self._loop)
+        self.tpc_recover = async_to_sync(self.atpc_recover, self._loop)
+        super(Connection, self).__init__(*args, **kwargs)
 
     # DBAPI Extension: supply exceptions as attributes on the connection
     Warning = property(lambda self: self._getError(Warning))
@@ -654,7 +704,7 @@ class Connection(CoreConnection):
         """
         return Cursor(self)
 
-    async def commit(self):
+    async def acommit(self):
         """Commits the current database transaction.
 
         This function is part of the `DBAPI 2.0 specification
@@ -662,7 +712,8 @@ class Connection(CoreConnection):
         """
         await self.execute_unnamed("commit")
 
-    async def rollback(self):
+
+    async def arollback(self):
         """Rolls back the current database transaction.
 
         This function is part of the `DBAPI 2.0 specification
@@ -672,6 +723,7 @@ class Connection(CoreConnection):
             return
         await self.execute_unnamed("rollback")
 
+
     def xid(self, format_id, global_transaction_id, branch_qualifier):
         """Create a Transaction IDs (only global_transaction_id is used in pg)
         format_id and branch_qualifier are not used in postgres
@@ -680,7 +732,7 @@ class Connection(CoreConnection):
         (format_id, global_transaction_id, branch_qualifier)"""
         return (format_id, global_transaction_id, branch_qualifier)
 
-    async def tpc_begin(self, xid):
+    async def atpc_begin(self, xid):
         """Begins a TPC transaction with the given transaction ID xid.
 
         This method should be called outside of a transaction (i.e. nothing may
@@ -697,7 +749,8 @@ class Connection(CoreConnection):
         if self.autocommit:
             await self.execute_unnamed("begin transaction")
 
-    async def tpc_prepare(self):
+
+    async def atpc_prepare(self):
         """Performs the first phase of a transaction started with .tpc_begin().
         A ProgrammingError is be raised if this method is called outside of a
         TPC transaction.
@@ -710,7 +763,7 @@ class Connection(CoreConnection):
         """
         await self.execute_unnamed("PREPARE TRANSACTION '%s';" % (self._xid[1],))
 
-    async def tpc_commit(self, xid=None):
+    async def atpc_commit(self, xid=None):
         """When called with no arguments, .tpc_commit() commits a TPC
         transaction previously prepared with .tpc_prepare().
 
@@ -737,16 +790,17 @@ class Connection(CoreConnection):
         try:
             previous_autocommit_mode = self.autocommit
             self.autocommit = True
-            if xid in await self.tpc_recover():
+            if xid in await self.atpc_recover():
                 await self.execute_unnamed("COMMIT PREPARED '%s';" % (xid[1],))
             else:
                 # a single-phase commit
-                await self.commit()
+                await self.acommit()
         finally:
             self.autocommit = previous_autocommit_mode
         self._xid = None
 
-    async def tpc_rollback(self, xid=None):
+
+    async def atpc_rollback(self, xid=None):
         """When called with no arguments, .tpc_rollback() rolls back a TPC
         transaction. It may be called before or after .tpc_prepare().
 
@@ -771,17 +825,18 @@ class Connection(CoreConnection):
         try:
             previous_autocommit_mode = self.autocommit
             self.autocommit = True
-            if xid in await self.tpc_recover():
+            if xid in await self.atpc_recover():
                 # a two-phase rollback
                 await self.execute_unnamed("ROLLBACK PREPARED '%s';" % (xid[1],))
             else:
                 # a single-phase rollback
-                await self.rollback()
+                await self.arollback()
         finally:
             self.autocommit = previous_autocommit_mode
         self._xid = None
 
-    async def tpc_recover(self):
+
+    async def atpc_recover(self):
         """Returns a list of pending transaction IDs suitable for use with
         .tpc_commit(xid) or .tpc_rollback(xid).
 
@@ -792,12 +847,14 @@ class Connection(CoreConnection):
             previous_autocommit_mode = self.autocommit
             self.autocommit = True
             curs = self.cursor()
-            await curs.execute("select gid FROM pg_prepared_xacts")
+            await curs.aexecute("select gid FROM pg_prepared_xacts")
             return [self.xid(0, row[0], "") for row in curs.fetchall()]
         finally:
             self.autocommit = previous_autocommit_mode
 
 
+
+
 class Warning(Exception):
     """Generic exception raised for important database warnings like data
     truncations.  This exception is not currently used by pgwasm.
